/*
    SPDX-FileCopyrightText: 2026 Kristen McWilliam <kristen@kde.org>

    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
*/

#include "overlaytrigger.h"

OverlayTrigger::OverlayTrigger(QObject *parent)
    : QObject(parent)
{
}

#include "moc_overlaytrigger.cpp"
